#include <private/colormapchart_p.h>
#include <QtCharts/QColorMapSeries>
#include <private/qcolormapseries_p.h>
#include <private/chartpresenter_p.h>
#include <private/abstractdomain_p.h>
#include <private/chartdataset_p.h>
#include <private/qabstractaxis_p.h>
#include <QtGui/QPainter>


QT_CHARTS_BEGIN_NAMESPACE

ColorMapChart::ColorMapChart(QColorMapSeries *series, QGraphicsItem *item)
    : ChartItem(series->d_func(), item),
      m_series(series),
      m_dirty(true)
{
    //    QObject::connect(series, SIGNAL(pointReplaced(int)), this, SLOT(handlePointReplaced(int)));
    //    QObject::connect(series, SIGNAL(pointsReplaced()), this, SLOT(handlePointsReplaced()));
    //    QObject::connect(series, SIGNAL(pointAdded(int)), this, SLOT(handlePointAdded(int)));
    //    QObject::connect(series, SIGNAL(pointRemoved(int)), this, SLOT(handlePointRemoved(int)));
    //    QObject::connect(series, SIGNAL(pointsRemoved(int, int)), this, SLOT(handlePointsRemoved(int, int)));
    //    QObject::connect(this, SIGNAL(clicked(Point3D)), series, SIGNAL(clicked(Point3D)));
    //    QObject::connect(this, SIGNAL(hovered(Point3D,bool)), series, SIGNAL(hovered(Point3D,bool)));
    //    QObject::connect(this, SIGNAL(pressed(Point3D)), series, SIGNAL(pressed(Point3D)));
    //    QObject::connect(this, SIGNAL(released(Point3D)), series, SIGNAL(released(Point3D)));
    //    QObject::connect(this, SIGNAL(doubleClicked(Point3D)), series, SIGNAL(doubleClicked(Point3D)));

    m_abscissTable = new QMap<int, int>() ;
    m_ordinateTable = new QMap<int, int>() ;
    m_coordinateToColorTable = new QMap<QPair<int, int>,uint>();
}

void ColorMapChart::setDirty(bool dirty)
{
    m_dirty = dirty;
}

QRectF ColorMapChart::boundingRect() const
{
    return m_rect;
}

void ColorMapChart::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(widget)
    Q_UNUSED(option)
    static double debug=0.0;
    m_coordinateToColorTable->clear();
    m_abscissTable->clear();
    m_ordinateTable->clear();

    QRectF clipRect = QRectF(QPointF(0, 0), domain()->size());
    painter->setClipRect(clipRect);

    QRectF plotAreaRect = m_series->chart()->plotArea();
    QImage colorMapImage(plotAreaRect.width(),plotAreaRect.height(),QImage::Format_RGB32);
    //http://doc.qt.io/qt-4.8/qimage.html#details :Warning: This will create a QImage with uninitialized data. Call fill() to fill the image with an appropriate pixel value before drawing onto it with QPainter.
    colorMapImage.fill(QColor(Qt::white).rgb());

    ColorMapDataPart * grid = m_series->getUniformGrid(0,0,plotAreaRect.width(),plotAreaRect.height(),QColorMapSeries::MeanPixel);



    double maxZ = m_series->maxZ();
    double minZ = m_series->minZ();
    double rangeZ = maxZ - minZ;

    QLinearGradient gradient = createColorMapGradient(Rainbow);
    QGradientStops colorStops = gradient.stops();
QVector<QRgb> *values = new QVector<QRgb>();

    for(int i=0;i<colorMapImage.width();i++)
    {
        for(int j=0;j<colorMapImage.height();j++)
        {
            double value = grid->dataSeries().at(i+j*colorMapImage.width());
            double pix=((value-minZ)/rangeZ);
            for(int k =0;k<colorStops.size()-1;k++)
            {
                QGradientStop lowerBound = colorStops.at(k);
                QGradientStop upperBound = colorStops.at(k+1);
                if(pix >= lowerBound.first && pix < upperBound.first)
                {
                    double ratio = (pix-lowerBound.first)/(upperBound.first - lowerBound.first);
                    int red      = (int)(ratio*lowerBound.second.red() + (1-ratio)*upperBound.second.red());
                    int green = (int)(ratio*lowerBound.second.green() + (1-ratio)*upperBound.second.green());
                    int blue   = (int)(ratio*lowerBound.second.blue() + (1-ratio)*upperBound.second.blue());
                    colorMapImage.setPixel(i,j,qRgb(red, green, blue));
                    values->append(qRgb(red, green, blue));
                    break;
                }
                else
                {
                    if(k==colorStops.size()-2)
                    {
                        colorMapImage.setPixel(i,j,qRgb(colorStops.at(colorStops.size()-1).second.red(), colorStops.at(colorStops.size()-1).second.green(), colorStops.at(colorStops.size()-1).second.blue()));
                         values->append(qRgb(colorStops.at(colorStops.size()-1).second.red(), colorStops.at(colorStops.size()-1).second.green(), colorStops.at(colorStops.size()-1).second.blue()));
                    }
                }
            }
        }
    }
    painter->drawImage(clipRect,colorMapImage);
    update();

}

/*!
    Returns the predefined QLinearGradient corresponding to the \a gradientType passed as argument.
*/
QLinearGradient ColorMapChart::createColorMapGradient(GradientType gradientType)
{
    QLinearGradient gradient(0,0,1,100);
    switch(gradientType)
    {
    case Rainbow :
        gradient.setColorAt(1.0,Qt::darkRed);
        gradient.setColorAt(0.8,Qt::red);
        gradient.setColorAt(0.6,Qt::yellow);
        gradient.setColorAt(0.4,Qt::green);
        gradient.setColorAt(0.2,Qt::cyan);
        gradient.setColorAt(0.0,Qt::blue);
        break;
    case CyclingRainbow :
        gradient.setColorAt(1.0,Qt::red);
        gradient.setColorAt(0.8,Qt::yellow);
        gradient.setColorAt(0.6,Qt::green);
        gradient.setColorAt(0.4,Qt::cyan);
        gradient.setColorAt(0.2,Qt::blue);
        gradient.setColorAt(0.0,Qt::magenta);
        break;
    case Grey :
        gradient.setColorAt(1.0, Qt::black);
        gradient.setColorAt(0.0, Qt::white);
        break;
    default:
        break;
    }
return gradient;
}




//handlers

void ColorMapChart::handlePointAdded(int index)
{

}

void ColorMapChart::handlePointRemoved(int index)
{

}

void ColorMapChart::handlePointsRemoved(int index, int count)
{

}

void ColorMapChart::handlePointReplaced(int index)
{

}

void ColorMapChart::handlePointsReplaced()
{

}

void ColorMapChart::handleDomainUpdated()
{

}

bool ColorMapChart::isEmpty()
{

}


#include "moc_colormapchart_p.cpp"

QT_CHARTS_END_NAMESPACE
